@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -134,6 +134,12 @@ public class ProGuardTask extends ConfigurationTask
     }
 
 
+    public void setMergeinterfacesaggressively(boolean mergeinterfacesaggressively)
+    {
+        configuration.mergeInterfacesAggressively = mergeinterfacesaggressively;
+    }
+
+
     public void setObfuscate(boolean obfuscate)
     {
         configuration.obfuscate = obfuscate;
@@ -158,6 +164,18 @@ public class ProGuardTask extends ConfigurationTask
     }
 
 
+    public void setClassobfuscationdictionary(File classObfuscationDictionary)
+    {
+        configuration.classObfuscationDictionary = resolvedFile(classObfuscationDictionary);
+    }
+
+
+    public void setPackageobfuscationdictionary(File packageObfuscationDictionary)
+    {
+        configuration.packageObfuscationDictionary = resolvedFile(packageObfuscationDictionary);
+    }
+
+
     public void setOverloadaggressively(boolean overloadAggressively)
     {
         configuration.overloadAggressively = overloadAggressively;
