@@ -2,21 +2,21 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
  *
- * This library is free software; you can redistribute it and/or modify it
+ * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
  * Software Foundation; either version 2 of the License, or (at your option)
  * any later version.
  *
- * This library is distributed in the hope that it will be useful, but WITHOUT
+ * This program is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
- * for more details.
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+ * more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 package proguard.classfile.attribute.annotation;
 
@@ -50,6 +50,27 @@ public class Annotation implements VisitorAccepter
     public Object visitorInfo;
 
 
+    /**
+     * Creates an uninitialized Annotation.
+     */
+    public Annotation()
+    {
+    }
+
+
+    /**
+     * Creates an initialized Annotation.
+     */
+    public Annotation(int            u2typeIndex,
+                      int            u2elementValuesCount,
+                      ElementValue[] elementValues)
+    {
+        this.u2typeIndex          = u2typeIndex;
+        this.u2elementValuesCount = u2elementValuesCount;
+        this.elementValues        = elementValues;
+    }
+
+
     /**
      * Returns the type.
      */
