@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -31,6 +31,16 @@ public class TopValue extends Category1Value
 {
     // Implementations for Value.
 
+    public boolean isSpecific()
+    {
+        return true;
+    }
+
+    public boolean isParticular()
+    {
+        return true;
+    }
+
     public final Value generalize(Value other)
     {
         return this.getClass() == other.getClass() ? this : null;
