@@ -2,14 +2,13 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
  * Software Foundation; either version 2 of the License, or (at your option)
  * any later version.
  *
- *
  * This program is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
@@ -29,7 +28,7 @@ import proguard.classfile.editor.*;
 import proguard.classfile.instruction.visitor.*;
 import proguard.classfile.util.MethodLinker;
 import proguard.classfile.visitor.*;
-import proguard.evaluation.value.SpecificValueFactory;
+import proguard.evaluation.value.*;
 import proguard.optimize.evaluation.*;
 import proguard.optimize.info.*;
 import proguard.optimize.peephole.*;
@@ -70,27 +69,28 @@ public class Optimizer
         }
 
         // Create counters to count the numbers of optimizations.
-        ClassCounter       singleImplementationCounter = new ClassCounter();
-        ClassCounter       finalClassCounter           = new ClassCounter();
-        MemberCounter      finalMethodCounter          = new MemberCounter();
-        MemberCounter      privateFieldCounter         = new MemberCounter();
-        MemberCounter      privateMethodCounter        = new MemberCounter();
-        MemberCounter      staticMethodCounter         = new MemberCounter();
-        MemberCounter      writeOnlyFieldCounter       = new MemberCounter();
-        MemberCounter      constantFieldCounter        = new MemberCounter();
-        MemberCounter      constantMethodCounter       = new MemberCounter();
-        MemberCounter      descriptorShrinkCounter     = new MemberCounter();
-        MemberCounter      initializerFixCounter       = new MemberCounter();
-        MemberCounter      parameterShrinkCounter      = new MemberCounter();
-        MemberCounter      variableShrinkCounter       = new MemberCounter();
-        ExceptionCounter   exceptionCounter            = new ExceptionCounter();
-        InstructionCounter inliningCounter             = new InstructionCounter();
-        InstructionCounter commonCodeCounter           = new InstructionCounter();
-        InstructionCounter pushCounter                 = new InstructionCounter();
-        InstructionCounter branchCounter               = new InstructionCounter();
-        InstructionCounter deletedCounter              = new InstructionCounter();
-        InstructionCounter addedCounter                = new InstructionCounter();
-        InstructionCounter peepholeCounter             = new InstructionCounter();
+        ClassCounter       classMergeCounter       = new ClassCounter();
+        ClassCounter       finalClassCounter       = new ClassCounter();
+        MemberCounter      finalMethodCounter      = new MemberCounter();
+        MemberCounter      privateFieldCounter     = new MemberCounter();
+        MemberCounter      privateMethodCounter    = new MemberCounter();
+        MemberCounter      staticMethodCounter     = new MemberCounter();
+        MemberCounter      writeOnlyFieldCounter   = new MemberCounter();
+        MemberCounter      constantFieldCounter    = new MemberCounter();
+        MemberCounter      constantMethodCounter   = new MemberCounter();
+        MemberCounter      descriptorShrinkCounter = new MemberCounter();
+        MemberCounter      initializerFixCounter   = new MemberCounter();
+        MemberCounter      parameterShrinkCounter  = new MemberCounter();
+        MemberCounter      variableShrinkCounter   = new MemberCounter();
+        ExceptionCounter   exceptionCounter        = new ExceptionCounter();
+        InstructionCounter inliningCounter         = new InstructionCounter();
+        InstructionCounter tailRecursionCounter    = new InstructionCounter();
+        InstructionCounter commonCodeCounter       = new InstructionCounter();
+        InstructionCounter pushCounter             = new InstructionCounter();
+        InstructionCounter branchCounter           = new InstructionCounter();
+        InstructionCounter deletedCounter          = new InstructionCounter();
+        InstructionCounter addedCounter            = new InstructionCounter();
+        InstructionCounter peepholeCounter         = new InstructionCounter();
 
         // Clean up any old visitor info.
         programClassPool.classesAccept(new ClassCleaner());
@@ -129,8 +129,10 @@ public class Optimizer
         programClassPool.classesAccept(new AllConstantVisitor(
                                        new ClassForNameClassVisitor(keepMarker)));
 
-        // Attach some optimization info to all class members, so it can be
-        // filled out later.
+        // Attach some optimization info to all classes and class members, so
+        // it can be filled out later.
+        programClassPool.classesAccept(new ClassOptimizationInfoSetter());
+
         programClassPool.classesAccept(new AllMemberVisitor(
                                        new MemberOptimizationInfoSetter()));
 
@@ -148,9 +150,6 @@ public class Optimizer
             libraryClassPool.accept(noClassPoolvisitor);
         }
 
-        // Mark all interfaces that have single implementations.
-        programClassPool.classesAccept(new SingleImplementationMarker(configuration.allowAccessModification, singleImplementationCounter));
-
         // Make classes and methods final, as far as possible.
         programClassPool.classesAccept(new ClassFinalizer(finalClassCounter, finalMethodCounter));
 
@@ -158,11 +157,7 @@ public class Optimizer
         programClassPool.classesAccept(new AllMethodVisitor(
                                        new AllAttributeVisitor(
                                        new AllInstructionVisitor(
-                                       new MultiInstructionVisitor(
-                                       new InstructionVisitor[]
-                                       {
-                                           new ReadWriteFieldMarker(),
-                                       })))));
+                                       new ReadWriteFieldMarker()))));
 
         // Mark all used parameters, including the 'this' parameters.
         programClassPool.classesAccept(new AllMethodVisitor(
@@ -177,9 +172,11 @@ public class Optimizer
 
         // Perform partial evaluation for filling out fields, method parameters,
         // and method return values.
+        ValueFactory valueFactory = new IdentifiedValueFactory();
+
         programClassPool.classesAccept(new AllMethodVisitor(
                                        new AllAttributeVisitor(
-                                       new PartialEvaluator(new SpecificValueFactory(), new StoringInvocationUnit(), false))));
+                                       new PartialEvaluator(valueFactory, new StoringInvocationUnit(valueFactory), false))));
 
         // Simplify based on partial evaluation.
         // Also remove unused parameters from the stack before method invocations,
@@ -188,7 +185,7 @@ public class Optimizer
         programClassPool.classesAccept(new AllMethodVisitor(
                                        new AllAttributeVisitor(
                                        new EvaluationSimplifier(
-                                       new PartialEvaluator(new SpecificValueFactory(), new LoadingInvocationUnit(), false),
+                                       new PartialEvaluator(valueFactory, new LoadingInvocationUnit(valueFactory), false),
                                        pushCounter, branchCounter, deletedCounter, addedCounter))));
 
         // Shrink the parameters in the method descriptors.
@@ -237,12 +234,6 @@ public class Optimizer
 //                                       new OptimizationInfoMemberFilter(
 //                                       new ClassReferenceFixer(true))));
 
-        // Inline interfaces with single implementations.
-        programClassPool.classesAccept(new SingleImplementationInliner());
-
-        // Restore the interface references from these single implementations.
-        programClassPool.classesAccept(new SingleImplementationFixer());
-
         if (configuration.allowAccessModification)
         {
             // Fix the access flags of referenced classes and class members,
@@ -251,31 +242,81 @@ public class Optimizer
                                            new AccessFixer()));
         }
 
-        // Fix all references to classes, for SingleImplementationInliner.
-        programClassPool.classesAccept(new ClassReferenceFixer(true));
-
-        // Fix all references to class members, for SingleImplementationInliner.
-        programClassPool.classesAccept(new MemberReferenceFixer());
-
+        // Mark all classes with package visible members.
+        // Mark all exception catches of methods.
         // Count all method invocations.
         // Mark super invocations and other access of methods.
-        // Mark all exception catches of methods.
-        programClassPool.classesAccept(new AllMethodVisitor(
-                                       new AllAttributeVisitor(
-                                       new MultiAttributeVisitor(
-                                       new AttributeVisitor[]
+        programClassPool.classesAccept(new MultiClassVisitor(
+                                       new ClassVisitor[]
                                        {
-                                           new AllInstructionVisitor(
-                                           new MultiInstructionVisitor(
-                                           new InstructionVisitor[]
+                                           new AllConstantVisitor(
+                                           new PackageVisibleMemberInvokingClassMarker()),
+                                           new AllMethodVisitor(
+                                           new MultiMemberVisitor(
+                                           new MemberVisitor[]
                                            {
-                                               new MethodInvocationMarker(),
-                                               new SuperInvocationMarker(),
-                                               new BackwardBranchMarker(),
-                                               new AccessMethodMarker(),
+                                               new PackageVisibleMemberContainingClassMarker(),
+                                               new AllAttributeVisitor(
+                                               new MultiAttributeVisitor(
+                                               new AttributeVisitor[]
+                                               {
+                                                   new CatchExceptionMarker(),
+                                                   new AllInstructionVisitor(
+                                                   new MultiInstructionVisitor(
+                                                   new InstructionVisitor[]
+                                                   {
+                                                       new InstantiationClassMarker(),
+                                                       new InstanceofClassMarker(),
+                                                       new DotClassMarker(),
+                                                       new MethodInvocationMarker(),
+                                                       new SuperInvocationMarker(),
+                                                       new BackwardBranchMarker(),
+                                                       new AccessMethodMarker(),
+                                                   })),
+                                               })),
                                            })),
-                                           new CatchExceptionMarker(),
-                                       }))));
+                                       }));
+
+        // Tweak the descriptors of duplicate initializers.
+        programClassPool.classesAccept(new AllMethodVisitor(
+                                       new DuplicateInitializerFixer(initializerFixCounter)));
+
+        if (initializerFixCounter.getCount() > 0)
+        {
+            // Fix all invocations of tweaked initializers.
+            programClassPool.classesAccept(new AllMethodVisitor(
+                                           new AllAttributeVisitor(
+                                           new DuplicateInitializerInvocationFixer(addedCounter))));
+
+            // Fix all references to tweaked initializers.
+            programClassPool.classesAccept(new MemberReferenceFixer());
+        }
+
+        // Merge classes into their superclasses or interfaces.
+        programClassPool.classesAccept(new VerticalClassMerger(configuration.allowAccessModification,
+                                                               configuration.mergeInterfacesAggressively,
+                                                               classMergeCounter));
+
+        // Merge classes into their sibling classes.
+        programClassPool.classesAccept(new HorizontalClassMerger(configuration.allowAccessModification,
+                                                                 configuration.mergeInterfacesAggressively,
+                                                                 classMergeCounter));
+
+        // Clean up inner class attributes to avoid loops.
+        programClassPool.classesAccept(new RetargetedInnerClassAttributeRemover());
+
+        // Update references to merged classes.
+        programClassPool.classesAccept(new TargetClassChanger());
+        programClassPool.classesAccept(new ClassReferenceFixer(true));
+        programClassPool.classesAccept(new MemberReferenceFixer());
+
+        if (configuration.allowAccessModification)
+        {
+            // Fix the access flags of referenced merged classes and their class
+            // members.
+            programClassPool.classesAccept(new AllConstantVisitor(
+                                           new AccessFixer()));
+        }
 
         // Inline methods that are only invoked once.
         programClassPool.classesAccept(new AllMethodVisitor(
@@ -287,6 +328,11 @@ public class Optimizer
                                        new AllAttributeVisitor(
                                        new MethodInliner(configuration.microEdition, configuration.allowAccessModification, false, inliningCounter))));
 
+        // Simplify tail recursion calls.
+        programClassPool.classesAccept(new AllMethodVisitor(
+                                       new AllAttributeVisitor(
+                                       new TailRecursionSimplifier(tailRecursionCounter))));
+
         // Mark all class members that can not be made private.
         programClassPool.classesAccept(new NonPrivateMemberMarker());
 
@@ -306,7 +352,7 @@ public class Optimizer
         }
 
         // Fix invocations of methods that have become non-abstract or private,
-        // for SingleImplementationInliner, MemberPrivatizer, AccessFixer.
+        // for ClassMerger, MemberPrivatizer, AccessFixer.
         programClassPool.classesAccept(new AllMemberVisitor(
                                        new AllAttributeVisitor(
                                        new MethodInvocationFixer())));
@@ -335,24 +381,6 @@ public class Optimizer
                                            new GotoReturnReplacer(                              codeAttributeEditor, peepholeCounter),
                                        })))));
 
-        // Tweak the descriptors of duplicate initializers.
-        DuplicateInitializerFixer duplicateInitializerFixer =
-            new DuplicateInitializerFixer(initializerFixCounter);
-
-        programClassPool.classesAccept(new AllMethodVisitor(
-                                       duplicateInitializerFixer));
-
-        if (initializerFixCounter.getCount() > 0)
-        {
-            // Fix all invocations of tweaked initializers.
-            programClassPool.classesAccept(new AllMethodVisitor(
-                                           new AllAttributeVisitor(
-                                           new DuplicateInitializerInvocationFixer())));
-
-            // Fix all references to tweaked initializers.
-            programClassPool.classesAccept(new MemberReferenceFixer());
-        }
-
         // Remove unnecessary exception handlers.
         programClassPool.classesAccept(new AllMethodVisitor(
                                        new AllAttributeVisitor(
@@ -373,67 +401,70 @@ public class Optimizer
                                        new AllAttributeVisitor(
                                        new VariableOptimizer(!configuration.microEdition))));
 
-        int singleImplementationCount = singleImplementationCounter.getCount();
-        int finalClassCount           = finalClassCounter          .getCount();
-        int privateFieldCount         = privateFieldCounter        .getCount();
-        int privateMethodCount        = privateMethodCounter       .getCount();
-        int staticMethodCount         = staticMethodCounter        .getCount();
-        int finalMethodCount          = finalMethodCounter         .getCount();
-        int writeOnlyFieldCount       = writeOnlyFieldCounter      .getCount();
-        int constantFieldCount        = constantFieldCounter       .getCount();
-        int constantMethodCount       = constantMethodCounter      .getCount();
-        int descriptorShrinkCount     = descriptorShrinkCounter    .getCount() - initializerFixCounter.getCount();
-        int parameterShrinkCount      = parameterShrinkCounter     .getCount() - initializerFixCounter.getCount();
-        int variableShrinkCount       = variableShrinkCounter      .getCount();
-        int exceptionCount            = exceptionCounter           .getCount();
-        int inliningCount             = inliningCounter            .getCount();
-        int commonCodeCount           = commonCodeCounter          .getCount();
-        int pushCount                 = pushCounter                .getCount();
-        int branchCount               = branchCounter              .getCount();
-        int removedCount              = deletedCounter             .getCount() - addedCounter.getCount();
-        int peepholeCount             = peepholeCounter            .getCount();
+        int classMergeCound       = classMergeCounter      .getCount();
+        int finalClassCount       = finalClassCounter      .getCount();
+        int privateFieldCount     = privateFieldCounter    .getCount();
+        int privateMethodCount    = privateMethodCounter   .getCount();
+        int staticMethodCount     = staticMethodCounter    .getCount();
+        int finalMethodCount      = finalMethodCounter     .getCount();
+        int writeOnlyFieldCount   = writeOnlyFieldCounter  .getCount();
+        int constantFieldCount    = constantFieldCounter   .getCount();
+        int constantMethodCount   = constantMethodCounter  .getCount();
+        int descriptorShrinkCount = descriptorShrinkCounter.getCount() - initializerFixCounter.getCount();
+        int parameterShrinkCount  = parameterShrinkCounter .getCount() - initializerFixCounter.getCount();
+        int variableShrinkCount   = variableShrinkCounter  .getCount();
+        int exceptionCount        = exceptionCounter       .getCount();
+        int tailRecursionCount    = tailRecursionCounter   .getCount();
+        int inliningCount         = inliningCounter        .getCount();
+        int commonCodeCount       = commonCodeCounter      .getCount();
+        int pushCount             = pushCounter            .getCount();
+        int branchCount           = branchCounter          .getCount();
+        int removedCount          = deletedCounter         .getCount() - addedCounter.getCount();
+        int peepholeCount         = peepholeCounter        .getCount();
 
         if (configuration.verbose)
         {
-            System.out.println("  Number of inlined interfaces:             "+singleImplementationCount);
-            System.out.println("  Number of finalized classes:              "+finalClassCount);
-            System.out.println("  Number of privatized fields:              "+privateFieldCount);
-            System.out.println("  Number of privatized methods:             "+privateMethodCount);
-            System.out.println("  Number of staticized methods:             "+staticMethodCount);
-            System.out.println("  Number of finalized methods:              "+finalMethodCount);
-            System.out.println("  Number of removed write-only fields:      "+writeOnlyFieldCount);
-            System.out.println("  Number of inlined constant fields:        "+constantFieldCount);
-            System.out.println("  Number of inlined constant methods:       "+constantMethodCount);
-            System.out.println("  Number of simplified method declarations: "+descriptorShrinkCount);
-            System.out.println("  Number of removed parameters:             "+parameterShrinkCount);
-            System.out.println("  Number of removed local variables:        "+variableShrinkCount);
-            System.out.println("  Number of inlined method calls:           "+inliningCount);
-            System.out.println("  Number of removed exception blocks:       "+exceptionCount);
-            System.out.println("  Number of merged code blocks:             "+commonCodeCount);
-            System.out.println("  Number of simplified push instructions:   "+pushCount);
-            System.out.println("  Number of simplified branches:            "+branchCount);
-            System.out.println("  Number of removed instructions:           "+removedCount);
-            System.out.println("  Number of peephole optimizations:         "+peepholeCount);
+            System.out.println("  Number of merged classes:                  "+classMergeCound);
+            System.out.println("  Number of finalized classes:               "+finalClassCount);
+            System.out.println("  Number of privatized fields:               "+privateFieldCount);
+            System.out.println("  Number of privatized methods:              "+privateMethodCount);
+            System.out.println("  Number of staticized methods:              "+staticMethodCount);
+            System.out.println("  Number of finalized methods:               "+finalMethodCount);
+            System.out.println("  Number of removed write-only fields:       "+writeOnlyFieldCount);
+            System.out.println("  Number of inlined constant fields:         "+constantFieldCount);
+            System.out.println("  Number of inlined constant methods:        "+constantMethodCount);
+            System.out.println("  Number of simplified method declarations:  "+descriptorShrinkCount);
+            System.out.println("  Number of removed parameters:              "+parameterShrinkCount);
+            System.out.println("  Number of removed local variables:         "+variableShrinkCount);
+            System.out.println("  Number of inlined method calls:            "+inliningCount);
+            System.out.println("  Number of simplified tail recursion calls: "+tailRecursionCount);
+            System.out.println("  Number of removed exception blocks:        "+exceptionCount);
+            System.out.println("  Number of merged code blocks:              "+commonCodeCount);
+            System.out.println("  Number of simplified push instructions:    "+pushCount);
+            System.out.println("  Number of simplified branches:             "+branchCount);
+            System.out.println("  Number of removed instructions:            "+removedCount);
+            System.out.println("  Number of peephole optimizations:          "+peepholeCount);
         }
 
-        return singleImplementationCount > 0 ||
-               finalClassCount           > 0 ||
-               privateFieldCount         > 0 ||
-               privateMethodCount        > 0 ||
-               staticMethodCount         > 0 ||
-               finalMethodCount          > 0 ||
-               writeOnlyFieldCount       > 0 ||
-               constantFieldCount        > 0 ||
-               constantMethodCount       > 0 ||
-               descriptorShrinkCount     > 0 ||
-               parameterShrinkCount      > 0 ||
-               variableShrinkCount       > 0 ||
-               inliningCount             > 0 ||
-               exceptionCount            > 0 ||
-               commonCodeCount           > 0 ||
-               pushCount                 > 0 ||
-               branchCount               > 0 ||
-               removedCount              > 0 ||
-               peepholeCount             > 0;
+        return classMergeCound       > 0 ||
+               finalClassCount       > 0 ||
+               privateFieldCount     > 0 ||
+               privateMethodCount    > 0 ||
+               staticMethodCount     > 0 ||
+               finalMethodCount      > 0 ||
+               writeOnlyFieldCount   > 0 ||
+               constantFieldCount    > 0 ||
+               constantMethodCount   > 0 ||
+               descriptorShrinkCount > 0 ||
+               parameterShrinkCount  > 0 ||
+               variableShrinkCount   > 0 ||
+               inliningCount         > 0 ||
+               tailRecursionCount    > 0 ||
+               exceptionCount        > 0 ||
+               commonCodeCount       > 0 ||
+               pushCount             > 0 ||
+               branchCount           > 0 ||
+               removedCount          > 0 ||
+               peepholeCount         > 0;
     }
 }
